from collections import defaultdict
from typing import List, Dict

from socketio import AsyncServer

from server.game.manager import ClientManager
from server.game.message import MessageToPlayer
from server.model.game import Game
from server.model.players import Player, Announce


class LobbyManager(ClientManager):
    """ A ClientManager that handles a lobby, then orchestrates games.
    """

    def __init__(self, sio: AsyncServer):
        super().__init__()
        self.last_announces: Dict[Player, Announce] = {}
        self.lobby: Dict[Player, bool] = {}
        self.games: List[Game] = []
        self.sio = sio

    @property
    def nb_ready(self):
        return len(self.players_ready)

    @property
    def players_ready(self):
        return [k for k, v in self.lobby.items() if v]

    async def add_player(self, player: Player, is_ready: bool = False) -> None:
        self.lobby[player] = is_ready
        print(f"Added {player} to a lobby with {len(self.lobby)} players.")
        await self.sio.emit('messageChannel', "PONG")

    def wants_to_play(self, player: Player):
        self.lobby[player] = True
        print(f"{player} ready to play! ({self.nb_ready} people ready)")

    def announces(self, player: Player, announce: Announce):
        # FIXME: Call this message on incoming "ANNOUNCE"
        self.last_announces[player] = announce
        print(f"{player} ready to play! ({self.nb_ready} people ready)")

    def start_game(self, max_players=2):
        players = self.players_ready[:max_players]
        self.games.append(Game(players, manager=self))

        print(f"Game started : {' vs '.join([p.name for p in players])}")

    def send(self, to: Player, message: MessageToPlayer, extra=None):
        self.sio.send(message)
        pass
